import jsonschema
import pytest

from inventory_provider.config import CONFIG_SCHEMA


@pytest.fixture
def config():
    return {
        'redis': {
            'hostname': 'localhost',
            'port': 6379,
            'password': 'redis_password',
        },
        'ssh': {
            'username': 'test_username',
            'private-key': 'private_key_content',
            'known-hosts': 'known_hosts_content'
        },
        "nokia-ssh": {
            "username": "uSeR-NaMe",
            "password": "dummy-password"
        },
        'redis-databases': [0, 1, 2],
        'ims': {
            'api': 'ims_api',
            'username': 'ims_username',
            'password': 'ims_password',
        },
        'gws-direct': [],
        'nren-asn-map': [],
        'aai': {
            'discovery_endpoint_url': 'https://smaple.discovery.endpoint',
            'inventory_provider': {
                'client_id': 'sample-client-id',
                'secret': 'sample-secret'
            }
        },
        'orchestrator': {
            'url': 'https://orchestrator.url'
        }
    }


def test_config_validation(config):
    assert validate_config(config)


def test_config_validation_include_sentinel(config):
    config['sentinel'] = {  # noqa
        'hostname': 'localhost',
        'port': 26379,
        'password': 'redis_password',
        'name': 'mymaster'
    }
    assert not validate_config(config)
    # Configuration validation failed.
    # One of the redis and sentinel should exist in the config.

    del config['redis']
    assert validate_config(config)  # Configuration validation successful.


def test_config_validation_without_redis_password(config):
    # Password is optional.
    del config['redis']['password']
    assert validate_config(config)


def validate_config(config):
    try:
        jsonschema.validate(config, CONFIG_SCHEMA)
        return True  # Configuration validation successful.
    except jsonschema.exceptions.ValidationError:
        return False  # Configuration validation failed.
